/* Monte Carlo Method */

#include <stdio.h>
#include <stdlib.h>
#include <libc.h>
#include <math.h>
#include <handy.h>

#define  PI       3.141592653589793
#define  N        200
#define  INFLATION 1.05
#define  R        360.0
#define  W        200.0
#define  STEP     10.0
#define  WID      640.0

static int shot(int);
int cw,gw;

int main(void)
{
    int total, trycount, hit, i;
    double x, ratio;

    cw = HgWOpen(100.0, 280.0, R, R);
    gw = HgWOpen(160.0, 20.0, WID, W);
    HgWSetFillColor(cw, HgRGB(0.8, 0.5, 0.9));
    HgWCircleFill(cw, 0.0, 0.0, R, 0);
    HgWSetFillColor(gw, HG_BLACK);
    HgWSetFont(gw, HG_GB, 28.0);
    HgWText(gw, 50.0, 20.0, "モンテカルロ法による円周率の近似");
    HgWSetWidth(gw, 1.0);
    HgWLine(gw, 0.0, W/2.0, WID, W/2.0);
    srandom(getpid());
    for (i=0; i<20; i++)
        (void)random();

    total = trycount = N;
    hit = shot(trycount);
    ratio = (double)hit * 4.0 / (double)total;
    HgWSetFillColor(gw, HG_ORANGE);
    HgWSetWidth(gw, 2.0);
    x = 0.0;
    HgWMoveTo(gw, x, (ratio - (PI-0.125)) * 4*W);
    while (x < WID) {
        trycount *= INFLATION;
        hit += shot(trycount);  total += trycount;
        ratio = (double)hit * 4.0 / (double)total;
        HgWLineTo(gw, x += STEP, (ratio - (PI-0.125)) * 4*W);
        printf("%f = %d / %d\n", ratio, hit * 4, total);
    }
    (void) getchar();
    return 0;
}


#define frnd()    ((double)(random() & 0x3fffffff) / (double)0x3fffffff)

static int shot(int n)
{
    int i, h=0;
    double x,y;

    for (i=0; i<n; i++) {
        x = frnd(), y = frnd();
        if ((x*x + y*y) < 1.0) {
            h++;
            HgWSetFillColor(cw, HG_BLUE);
        }else
            HgWSetFillColor(cw, HG_RED);
        HgWCircleFill(cw, x*R, y*R, 1.2, 0);
    }
    return h;
}
