#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <handy.h>
#define Deg  (3.14159/180.0)

static double scale = 1.2;
static double eycolr[3] = { 0.7, 0.43, 0.21 };
static hgcolor skin;

static hgcolor makeBright(double v[], double rate, double alpha)
{
    double b[3];
    int i;
    for (i = 0; i < 3; i++)
        b[i] = 1.0 - (1.0 - v[i]) / rate;
    return HgRGBA(b[0], b[1], b[2], alpha);
}

int makeEye(int eyes, int work, int ex, int ey)
{
    HgWSetFillColor(work, HgRGB(eycolr[0], eycolr[1], eycolr[2]));
    HgWOvalFill(work, ex, ey, 80.0, 104.0, 0.0, 0);
    HgWSetFillColor(work, makeBright(eycolr, 1.4, 1.0));
    HgWOvalFill(work, ex, ey-43.0, 54.0, 43.0, 0.0, 0);
    HgWSetFillColor(work, makeBright(eycolr, 2.0, 1.0));
    HgWOvalFill(work, ex, ey-65.0, 36.0, 21.0, 0.0, 0);
    HgWSetWidth(work, 16.0*scale);
    HgWOval(work, ex, ey, 72.0, 96.0, 0.0);
    HgWSetFillColor(work, HG_BLACK);
    HgWOvalFill(work, ex, ey+4.0, 24.0, 30.0, 0.0, 0);

    HgWSetFillColor(work, HgRGBA(1.0, 1.0, 1.0, 0.8));
    HgWOvalFill(work, ex+46.0, ey+56.0, 22.0, 34.0, 16.0*Deg, 0);
    HgWCircleFill(work, ex-48.0, ey-58.0, 12.0, 0);

    HgWSetComposite(eyes, HG_BLEND_COPY);
    HgWSetFillColor(eyes, makeBright(eycolr, 4.0, 0.8));
    HgWOvalFill(eyes, ex, ey-20.0, 40.0, 50.0, 0.0, 0);
    HgWSetFillColor(eyes, HG_CLEAR);
    HgWOvalFill(eyes, ex, ey+4.0, 48.0, 54.0, 0.0, 0);

    HgWSetComposite(work, HG_BLEND_SOVER);
    HgWImageDraw(work, 0, 0, eyes);
    HgWImageDraw(eyes, 0, 0, work);
    return HgImageDupRect(eyes, 1.0, 0.0, ex-80, ey-104, 160, 208);
}

void mirror(int face)
{
    double ww, hh;
    HgWGetSize(face, &ww, &hh);
    HgWCoordinateEnable(face, 0);
    int dup = HgImageDupRect(face, 1.0,
        HG_FLIP_HORIZONTAL, 0, 0, ww/2.0, hh);
    HgWImageDraw(face, ww/2.0, 0, dup);
    HgImageUnload(dup);
    HgWCoordinateEnable(face, 1);
}

void makeFace(int face, double lx, double ly)
{
    double ww, hh;
    HgWGetSize(face, &ww, &hh);
    HgWCoordinateEnable(face, 0);
    HgWSetFillColor(face, skin);
    HgWBoxFill(face, 0, 0, ww, hh, 0);
    HgWCoordinateEnable(face, 1);
    HgWSetComposite(face, HG_BLEND_COPY);
    HgWSetFillColor(face, HG_BROWN);
    HgWOvalFill(face, lx-60.0, ly+102.0, 192.0, 81.0, 16.0 * Deg, 0);
    HgWSetFillColor(face, skin);
    HgWOvalFill(face, lx+5.0, ly+22.0, 268.0, 163.0, 0, 0);
    HgWSetFillColor(face, HG_BLACK);
    HgWOvalFill(face, lx+11.0, ly+52.0, 130.8, 98.9, 20.0 * Deg, 0);
    HgWSetFillColor(face, skin);
    HgWOvalFill(face, lx+118.0, ly+106.0, 16.0, 30.0, 29.0 * Deg, 0);
    HgWBoxFill(face, lx+108.0, ly-4.0, 38.0, 94.0, 0);
    HgWSetFillColor(face, HG_CLEAR);
    HgWOvalFill(face, lx+8.0, ly+36.0, 118.1, 93.1, 19.0 * Deg, 0);
    HgWOvalFill(face, lx+28.0, ly+36.5, 95.5, 85.0, 0, 0);
    HgWOvalFill(face, lx+30.0, ly+42.0, 95.5, 89.0, 0, 0);
    HgWSetFillColor(face, skin);
    HgWFanFill(face, lx+5.0, ly+128.0, 200.0, 237.4*Deg, 289*Deg, 0);
    HgWSetFillColor(face, HG_CLEAR);
    HgWFanFill(face, lx+5.0, ly+128.0, 180.0, 237.2*Deg, 290*Deg, 0);
    HgWSetWidth(face, 1.0*scale);
    HgWArc(face, lx+5.0, ly+128.0, 180.0, 255*Deg, 285*Deg);
    HgWArc(face, lx+40.0, ly+20.0, 150, 60.0 * Deg, 100.0 * Deg);
    HgWArc(face, lx+32.0, ly+20.0, 135, 50.0 * Deg, 80.0 * Deg);
    HgWArc(face, lx+32.0, ly+24.0, 120, 46.0 * Deg, 80.0 * Deg);
    HgWArc(face, lx+32.0, ly+47.0, 90, 35.0 * Deg, 80.0 * Deg);
    HgWArc(face, lx-100.0, ly+5.0, 120, 73.0 * Deg, 78.0 * Deg);
    HgWArc(face, lx-100.0, ly+5.0, 150, 64.0 * Deg, 70.0 * Deg);
    mirror(face);
}

void makeLid(int f[], double lx, double ly)
{
    int f1 = f[0];
    int f2 = f[1];
    HgWSetComposite(f1, HG_BLEND_COPY);
    HgWSetFillColor(f1, skin);
    HgWOvalFill(f1, lx+12, ly+62.0, 158, 120, 0.0, 0);
    // HgWBoxFill(f1, lx-122.0, ly-80.0, 265.0, 105.0, 0);
    HgLCopy(f2, f1);
    HgWSetFillColor(f1, HG_BLACK);
    HgWOvalFill(f1, lx+12.0, ly+40.0, 124, 82.0, 18.0 * Deg, 0);
    HgWSetFillColor(f1, skin);
    HgWOvalFill(f1, lx+122.0, ly+83.0, 10.0, 30.0, 41.0 * Deg, 0);
    HgWBoxFill(f1, lx+118.0, ly-4.0, 28.0, 74.0, 0);
    HgWSetFillColor(f1, HG_CLEAR);
    HgWOvalFill(f1, lx+15.0, ly+23.0, 120.0, 80.0, 16.0 * Deg, 0);
    HgWSetWidth(f1, 1.0*scale);
    HgWArc(f1, lx+52.0, ly+29.0, 89, 45.0 * Deg, 80.0 * Deg);
    HgWArc(f1, lx+55.0, ly+32.0, 77, 44.0 * Deg, 80.0 * Deg);
    HgWArc(f1, lx-100.0, ly-25.0, 120, 73.0 * Deg, 78.0 * Deg);
    HgWArc(f1, lx-100.0, ly-25.0, 150, 64.0 * Deg, 70.0 * Deg);

    int wk = HgWAddLayer(HgWindowOfLayer(f1));
    HgWSetWidth(wk, 1.0*scale);
    HgWSetComposite(wk, HG_BLEND_COPY);
    HgWSetFillColor(wk, skin);
    HgWFanFill(wk, lx+2.0, ly+138.0, 200.0, 235*Deg, 311*Deg, 0);
    HgWSetFillColor(wk, HG_CLEAR);
    HgWFanFill(wk, lx+2.0, ly+138.0, 178.0, 233.5*Deg, 312*Deg, 0);
    HgWSetComposite(f1, HG_BLEND_SOVER);
    HgWImageDraw(f1, 0, 0, wk);
    HgWArc(f1, lx+2.0, ly+138.0, 178.0, 235*Deg, 309.5*Deg);
    HgWSetFillColor(wk, HG_BLACK);
    HgWFanFill(wk, lx+2.0, ly+113.0, 159.0, 233.5*Deg, 312*Deg, 0);
    HgWSetFillColor(wk, HG_CLEAR);
    HgWFanFill(wk, lx+2.0, ly+138.0, 178.0, 233.5*Deg, 312*Deg, 0);
    HgWSetComposite(f2, HG_BLEND_SOVER);
    HgWImageDraw(f2, 0, 0, wk);
    HgLRemove(wk);
    mirror(f1);

    HgWSetWidth(f2, 1.0*scale);
    HgWArc(f2, lx-66.0, ly-17.0, 20.0, 260*Deg, 310*Deg);
    HgWArc(f2, lx-44.0, ly-27.0, 20.0, 270*Deg, 324*Deg);
    mirror(f2);
}

static const double ovalc = 0.8;
static const double radeye = 50.0;

int inEye(double x, double y)
{
    double d = x * x + y * y / (ovalc * ovalc);
    return ( d < radeye * radeye);
}

void getpos(double mx, double my,
    double *ex, double *ey, double cx, double cy)
{
    const double dv = 5.0;
    double x, y, tx, ty;
    if (inEye(mx - cx, my - cy)) {
        tx = mx, ty = my;
    }else {
        double wx = mx - cx;
        double wy = (my - cy) / ovalc;
        double d = radeye / sqrt(wx * wx + wy * wy);
        tx = cx + wx * d;
        ty = cy + wy * d * ovalc;
    }
    if (*ex < tx) {
        if ((x = *ex + dv) > tx) x = tx;
    }else {
        if ((x = *ex - dv) < tx) x = tx;
    }
    if (*ey < ty) {
        if ((y = *ey + dv) > ty) y = ty;
    }else {
        if ((y = *ey - dv) < ty) y = ty;
    }
    *ex = x;
    *ey = y;
}

static double blink(void)
{
    int n = random() % 9 + 2;
    return (double)n;
}

void look(doubleLayer dl, int eyelid[],
    double leftx, double rightx, double stdy, int pic)
{
    int lid = HgLSwitch(&dl);
    double lx = leftx, ly = stdy;
    double rx = rightx, ry = stdy;
    HgWImagePut(lid, lx, ly, pic, 1.0, 0);
    HgWImagePut(lid, rx, ry, pic, 1.0, 0);
    lid = HgLSwitch(&dl);
    HgWSetEventMask(0, HG_MOUSE_MOVE | HG_MOUSE_DOWN | HG_KEY_DOWN);
    srandom((unsigned int)time(NULL));
    HgSetAlarmTimer(blink());
    while (1) {
        hgevent *ev = HgEvent();
        if (ev->type == HG_MOUSE_MOVE) {
            double x, y;
            HgWTransWtoA(0, ev->x, ev->y, &x, &y);
            getpos(x, y, &lx, &ly, leftx, stdy);
            getpos(x, y, &rx, &ry, rightx, stdy);
            HgLClear(lid);
            HgWImagePut(lid, lx, ly, pic, 1.0, 0);
            HgWImagePut(lid, rx, ry, pic, 1.0, 0);
            lid = HgLSwitch(&dl);
        }else if (ev->type == HG_KEY_DOWN) {
            if (ev->ch == 'q' || ev->ch == 0x1b)
                return;
            HgLClear(lid);
            lx = leftx, rx = rightx, ly = ry = stdy;
            HgWImagePut(lid, lx, ly, pic, 1.0, 0);
            HgWImagePut(lid, rx, ry, pic, 1.0, 0);
            lid = HgLSwitch(&dl);
        }else { // Mouse Down or Timer
            HgLShow(eyelid[0], 1);
            HgSleep(0.1);
            HgLShow(eyelid[1], 1);
            HgSleep(0.5);
            HgLShow(eyelid[1], 0);
            HgSleep(0.1);
            HgLShow(eyelid[0], 0);
            HgSetAlarmTimer(blink());
        }
    }
}

int displayEyes(void)
{
    double width = 700;
    int eyepic;
    int face, eyelid[2], eyes[2];
    doubleLayer dl;
    double leftx = 153.0, stdy = 125.0;
    double rightx = width - leftx;
    HgOpen(width*scale, 320*scale);
    if (scale != 1.0)
        HgWCoordinate(0, 0, 0, scale);
    eyes[0] = HgWAddLayer(0);
    eyes[1] = HgWAddLayer(0);
    face = HgWAddLayer(0);
    eyelid[0] = HgWAddLayer(0);
    eyelid[1] = HgWAddLayer(0);
    HgLShow(eyes[0], 0);
    HgLShow(eyes[1], 0);
    HgLShow(eyelid[0], 0);
    HgLShow(eyelid[1], 0);
    //HgLPDFRecord(0);
    skin = HgRGB(0.96, 0.88, 0.81);
    makeFace(face, leftx-3.0, stdy - 35.0);
    makeLid(eyelid, leftx-3.0, stdy - 35.0);
    eyepic = makeEye(eyes[0], eyes[1], leftx + 1.0, stdy + 8.0);
    HgLClear(eyes[0]);
    HgLClear(eyes[1]);
    dl.display = eyes[0];
    dl.hidden = eyes[1];
    look(dl, eyelid, leftx, rightx, stdy, eyepic);
    //HgLPDFSave(0, "eyes.pdf");
    HgClose();
    return 0;
}

int main(int argc, char *argv[])
{
    if (argc < 2) {
        printf("Usage: ./a.out scale [options]\n");
        printf("Option: -r  Red eyes\n");
        printf("        -g  Green eyes\n");
        printf("        -b  Blue eyes\n");
        printf("        -B  Brown eyes (default)\n");
        printf("Example: ./a.out 0.8 -r\n");
        return 1;
    }
    scale = atof(argv[1]);
    if (scale > 5.0 || scale < 0.1) {
        printf("Illegal scale...\n");
        return 1;
    }
    if (argc > 2 && argv[2][0] == '-') {
        int i;
        for (i = 0; i < 3; i++)
            eycolr[i] = 0.3;
        switch (argv[2][1]) {
        case 'r': eycolr[0] = 0.8; break;
        case 'g': eycolr[1] = 0.8; break;
        case 'b': eycolr[2] = 0.8; break;
        case 'B':
        default:
            eycolr[2] = 0.8; break;
        }
    }
    return displayEyes();
}
