/*
 画像ファイルを読み込み、大きさや角度を指定して描画
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <handy.h>

const int Width = 1000;
const int Height = 800;
const int Obj = 50.0;       // 画像のおおよその大きさ

typedef struct {
    int      img;        // imageID
    double   x, y, dx, dy;
    double   angle, th;     // 角度
}ball;

int readImage(ball *g, const char *path, double scale)
{
    double x, y, ratio;
    int img = HgImageLoad(path);
    if (img < 0)
        return 0;
    HgImageSize(img, &x, &y);
    ratio = (Obj * 4.0) * scale / (x + y);
    g->img = HgImageDup(img, ratio, HG_FLIP_NONE);
    HgImageUnload(img);
    return 1;
}

int initBall(ball *g)
{
    g->x = (double)(random() % (Width/2)) + Width/4;
    g->y = (double)(random() % (Height/2)) + Height/4;
    /* (x, y) 座標位置はランダム */
    g->dx = (double)(random() % 15 + 2) / 50.0 * Obj;
    g->dy = (double)(random() % 15 + 2) / 50.0 * Obj;
    /* (x, y) 方向の移動速度 */
    g->angle = 0.0;
    double rnd = (double)(random() % 30 + 10) / 100.0;
    if (random() & 1) rnd *= -1.0;
    g->th = rnd;     /* 角速度 */
    return 1;
}

void draw(doubleLayer *wlayp, ball g[], int num)
{
    int i;
    double x, y, a;
    int lid = HgLSwitch(wlayp);
    HgLClear(lid);                 /* レイヤを消去 */
    for (i = 0; i < num; i++) {
        int flag = 0;
        HgWImagePut(lid, g[i].x, g[i].y,
                g[i].img, 1.0, g[i].angle); // イメージを描く
        y = g[i].y += g[i].dy;
        if ( y < Obj || y > (Height - Obj)) {
            g[i].dy *= -1.0;
            flag = 1;
        }
        x = g[i].x += g[i].dx;  
        if ( x < Obj || x > (Width - Obj)) {
            g[i].dx *= -1.0;
            flag = 1;
        }
        a = g[i].angle += g[i].th;
        if (a < 0.0) a += M_PI;
        else if (a > M_PI) a -= M_PI;
        if (flag) {
            g[i].th *= -1.0;
        }
    }
    HgSleep(0.05);                /* 少し待つ */
}

int mainWindow(void)
{
    int pic;
    double w, h, r1, r2, r, pw, ph;

    HgOpen(Width, Height);  /* 描画ウィンドウを開く */
    HgWSetTitle(0, "bounce 6"); /* 名前をつける */
    pic = HgImageLoad("ksu.jpg");
    if (pic < 0)
        return 0;
    HgImageSize(pic, &w, &h);
    r1 = w / Width;
    r2 = h / Height;
    r = r1 < r2 ? r1 : r2;
    pw = Width * r;
    ph = Height * r;
    HgImageDrawRect(0, 0, Width, Height, pic,
            (w - pw) / 2.0, (h - ph) / 2.0, pw, ph);
    HgSetFillColor(HgGrayA(1.0, 0.5));
    HgBoxFill(0, 0, Width, Height, 0);
    HgSetFont(HG_M, 16.0);
    HgSetColor(HG_WHITE);
    HgText(20.0, 10.0, "京都産業大学 コンピュータ理工学部");
    HgSetColor(HG_BLACK);
    HgText(18.0, 12.0, "京都産業大学 コンピュータ理工学部");
    return 0;
}

int main(void) {
    const int number = 15;
    int win = 0, seed, i, k;
    ball group[number];
    doubleLayer wlay;

    // printf("random seed = ");
    // scanf("%d",&seed);
    seed = (int)getpid();
    srandom(seed); 

    double scale = 0.9;
    for (i = 0, k = 0; i < number; ) {
        k += readImage(&group[i++], "apple.png", scale);
        k += readImage(&group[i++], "tomato.png", scale);
        k += readImage(&group[i++], "redball.png", scale);
        k += readImage(&group[i++], "blueball.png", scale);
        k += readImage(&group[i++], "remon.png", scale);
        scale += 0.1;
    }
    if (i != k) {
        printf("ERROR: can't read images\n");
        return 1;
    }
    for (i = 0; i < number; i++)
        initBall(&group[i]);

    win = mainWindow();
    wlay = HgWAddDoubleLayer(win);
    int m;
    for (m = 0; m < 1000; m++) {
        draw(&wlay, group, number);
    }
    HgClose();
    return 0;
}
