#include <stdio.h>
#include <handy.h>

#define MAXR 200.0 // 描画する円の最大半径
#define MAXNUM 30 // 保持できる円の最大数

static int make_color(double a) {
    int x;
    double t, c[3];
    if (a >= 1.0) a -= 1.0;
    else if (a < 0.0) a += 1.0;
    x = (int)( t = a * 3.0 );
    t -= (double)x;
    c[x] = 1.0;
    c[(x + 1) % 3] = t;
    c[(x + 2) % 3] = 1.0 - t;
    return HgRGB(c[0], c[1], c[2]);
}

int main() {
  int win, code, type, i, pos;
  double x[MAXNUM], y[MAXNUM], r[MAXNUM];
  doubleLayer wlay;
  hgsound sid = HgSoundWithName("Hero");
  
  win=HgOpen(400,400);
  HgSetWidth(2.0);
  wlay = HgWAddDoubleLayer(win);
  HgWSetEventMask(win, HG_MOUSE_DOWN | HG_KEY_DOWN);
  for(i=0; i<MAXNUM; i++) r[i]=0.0; // 描画する円の半径をゼロに
  pos=0; // まずはゼロポジションから待ち受け
  while( 1 ) {
    hgevent *ev;
    int lid = HgLSwitch(&wlay);
    HgLClear(lid); 
    ev = HgEventNonBlocking();
    if (ev) {
      switch( ev->type ) {
        case HG_KEY_DOWN: // キー入力だったらプログラム終了
          HgClose();
          return 0;
        case HG_MOUSE_DOWN:  // マウスクリックだった
          r[pos]=1.0; // 描画する円の半径を増やす
          x[pos]=ev->x; y[pos]=ev->y; // 値を保持
          pos=(pos+1)%MAXNUM; // ポジションを次に移動して待つ
          HgSoundPlay(sid);
          break;
      }
    }
    for(i=0; i<MAXNUM; i++) {
      if( r[i] > 0.0 ) {
        HgWSetColor(lid, make_color(r[i] / MAXR));
        HgWCircle(lid, x[i], y[i], r[i]); // 半径がゼロで無ければ描画
        r[i]+=1.0; // 径を大きくする
        y[i]-=0.03 * (r[i]-40.0); // 少しだけ上に進んで、以降下げる
        if( r[i] > MAXR ) r[i]=0.0; // 大きくなりすぎたら中止
      }
    }
    HgSleep(0.01);
  }
}
