#include <stdio.h>
#include <handy.h>

int openWindow(int img, double *winw, double *winh)
{
    double width, height;
    double szw, szh;
    double screenw, screenh;
    HgScreenSize(&screenw, &screenh);
    screenw = (int)(screenw * 0.8);
    screenh = (int)(screenh * 0.8);
    HgImageSize(img, &szw, &szh);
    if (szw > screenw || szh > screenh) {
        double r1 = screenw / szw;
        double r2 = screenh / szh;
        double r = r1 < r2 ? r1 : r2;
        width = (int)(szw * r + 0.5);
        height = (int)(szh * r + 0.5);
    }else {
        width = szw;
        height = szh;
    }
    (void)HgOpen(width, height);
    *winw = width;
    *winh = height;
    return 0;
}

int main(int argc, char **argv)
{
    int img;
    double width, height;
    doubleLayer dl;
    const double move = 8.0;

    if (argc <= 1) {
        fprintf(stderr, "Usage: a.out image-filename\n");
        return 1;
    }
    if ((img = HgImageLoad(argv[1])) == HG_ERROR) {
        fprintf(stderr, "can't open \"%s\"\n", argv[1]);
        return 1;
    }
    openWindow(img, &width, &height);
    dl = HgWAddDoubleLayer(0);
    HgWImageDrawRect(dl.display, 0, 0, width, height, img, 0, 0, 0, 0);

    double remain = width - move;
    for ( ; ; ) {
        HgWImageDrawRect(dl.hidden, move, 0, remain, height,
            dl.display, 0, 0, remain, height);
        HgWImageDrawRect(dl.hidden, 0, 0, move, height,
            dl.display, remain, 0, move, height);
        (void)HgLSwitch(&dl);
        HgSleep(0.04);
    }
    (void)HgClose();

    return 0;
}
